@@ -12,6 +12,7 @@ import (
 	"github.com/rfjakob/gocryptfs/internal/configfile"
 	"github.com/rfjakob/gocryptfs/internal/exitcodes"
 	"github.com/rfjakob/gocryptfs/internal/prefer_openssl"
+	"github.com/rfjakob/gocryptfs/internal/readpassword"
 	"github.com/rfjakob/gocryptfs/internal/stupidgcm"
 	"github.com/rfjakob/gocryptfs/internal/tlog"
 )
@@ -20,11 +21,15 @@ import (
 type argContainer struct {
 	debug, init, zerokey, fusedebug, openssl, passwd, fg, version,
 	plaintextnames, quiet, nosyslog, wpanic,
-	longnames, allow_other, ro, reverse, aessiv, nonempty, raw64,
+	longnames, allow_other, reverse, aessiv, nonempty, raw64,
 	noprealloc, speed, hkdf, serialize_reads, forcedecode, hh, info,
-	sharedstorage, devrandom bool
+	sharedstorage, devrandom, fsck, trezor bool
+	// Mount options with opposites
+	dev, nodev, suid, nosuid, exec, noexec, rw, ro bool
 	masterkey, mountpoint, cipherdir, cpuprofile, extpass,
 	memprofile, ko, passfile, ctlsock, fsname, force_owner, trace string
+	// For reverse mode, --exclude is available. It can be specified multiple times.
+	exclude multipleStrings
 	// Configuration file name override
 	config             string
 	notifypid, scryptn int
@@ -37,21 +42,33 @@ type argContainer struct {
 	_forceOwner *fuse.Owner
 }
 
+type multipleStrings []string
+
+func (s *multipleStrings) String() string {
+	s2 := []string(*s)
+	return fmt.Sprint(s2)
+}
+
+func (s *multipleStrings) Set(val string) error {
+	*s = append(*s, val)
+	return nil
+}
+
 var flagSet *flag.FlagSet
 
 // prefixOArgs transform options passed via "-o foo,bar" into regular options
 // like "-foo -bar" and prefixes them to the command line.
 // Testcases in TestPrefixOArgs().
-func prefixOArgs(osArgs []string) []string {
+func prefixOArgs(osArgs []string) ([]string, error) {
 	// Need at least 3, example: gocryptfs -o    foo,bar
 	//                               ^ 0    ^ 1    ^ 2
 	if len(osArgs) < 3 {
-		return osArgs
+		return osArgs, nil
 	}
 	// Passing "--" disables "-o" parsing. Ignore element 0 (program name).
 	for _, v := range osArgs[1:] {
 		if v == "--" {
-			return osArgs
+			return osArgs, nil
 		}
 	}
 	// Find and extract "-o foo,bar"
@@ -60,8 +77,7 @@ func prefixOArgs(osArgs []string) []string {
 		if osArgs[i] == "-o" {
 			// Last argument?
 			if i+1 >= len(osArgs) {
-				tlog.Fatal.Printf("The \"-o\" option requires an argument")
-				os.Exit(exitcodes.Usage)
+				return nil, fmt.Errorf("The \"-o\" option requires an argument")
 			}
 			oOpts = strings.Split(osArgs[i+1], ",")
 			// Skip over the arguments to "-o"
@@ -87,18 +103,22 @@ func prefixOArgs(osArgs []string) []string {
 	}
 	// Add other arguments
 	newArgs = append(newArgs, otherArgs...)
-	return newArgs
+	return newArgs, nil
 }
 
 // parseCliOpts - parse command line options (i.e. arguments that start with "-")
 func parseCliOpts() (args argContainer) {
-	os.Args = prefixOArgs(os.Args)
-
 	var err error
 	var opensslAuto string
 
+	os.Args, err = prefixOArgs(os.Args)
+	if err != nil {
+		tlog.Fatal.Println(err)
+		os.Exit(exitcodes.Usage)
+	}
+
 	flagSet = flag.NewFlagSet(tlog.ProgramName, flag.ContinueOnError)
-	flagSet.Usage = helpShort
+	flagSet.Usage = func() {}
 	flagSet.BoolVar(&args.debug, "d", false, "")
 	flagSet.BoolVar(&args.debug, "debug", false, "Enable debug output")
 	flagSet.BoolVar(&args.fusedebug, "fusedebug", false, "Enable fuse library debug output")
@@ -118,7 +138,6 @@ func parseCliOpts() (args argContainer) {
 	flagSet.BoolVar(&args.longnames, "longnames", true, "Store names longer than 176 bytes in extra files")
 	flagSet.BoolVar(&args.allow_other, "allow_other", false, "Allow other users to access the filesystem. "+
 		"Only works if user_allow_other is set in /etc/fuse.conf.")
-	flagSet.BoolVar(&args.ro, "ro", false, "Mount the filesystem read-only")
 	flagSet.BoolVar(&args.reverse, "reverse", false, "Reverse mode")
 	flagSet.BoolVar(&args.aessiv, "aessiv", false, "AES-SIV encryption")
 	flagSet.BoolVar(&args.nonempty, "nonempty", false, "Allow mounting over non-empty directories")
@@ -133,6 +152,21 @@ func parseCliOpts() (args argContainer) {
 	flagSet.BoolVar(&args.info, "info", false, "Display information about CIPHERDIR")
 	flagSet.BoolVar(&args.sharedstorage, "sharedstorage", false, "Make concurrent access to a shared CIPHERDIR safer")
 	flagSet.BoolVar(&args.devrandom, "devrandom", false, "Use /dev/random for generating master key")
+	flagSet.BoolVar(&args.fsck, "fsck", false, "Run a filesystem check on CIPHERDIR")
+	if readpassword.TrezorSupport {
+		flagSet.BoolVar(&args.trezor, "trezor", false, "Protect the masterkey using a SatoshiLabs Trezor instead of a password")
+	}
+
+	// Mount options with opposites
+	flagSet.BoolVar(&args.dev, "dev", false, "Allow device files")
+	flagSet.BoolVar(&args.nodev, "nodev", false, "Deny device files")
+	flagSet.BoolVar(&args.suid, "suid", false, "Allow suid binaries")
+	flagSet.BoolVar(&args.nosuid, "nosuid", false, "Deny suid binaries")
+	flagSet.BoolVar(&args.exec, "exec", false, "Allow executables")
+	flagSet.BoolVar(&args.noexec, "noexec", false, "Deny executables")
+	flagSet.BoolVar(&args.rw, "rw", false, "Mount the filesystem read-write")
+	flagSet.BoolVar(&args.ro, "ro", false, "Mount the filesystem read-only")
+
 	flagSet.StringVar(&args.masterkey, "masterkey", "", "Mount with explicit master key")
 	flagSet.StringVar(&args.cpuprofile, "cpuprofile", "", "Write cpu profile to specified file")
 	flagSet.StringVar(&args.memprofile, "memprofile", "", "Write memory profile to specified file")
@@ -144,26 +178,25 @@ func parseCliOpts() (args argContainer) {
 	flagSet.StringVar(&args.fsname, "fsname", "", "Override the filesystem name")
 	flagSet.StringVar(&args.force_owner, "force_owner", "", "uid:gid pair to coerce ownership")
 	flagSet.StringVar(&args.trace, "trace", "", "Write execution trace to file")
+
+	// -e, --exclude
+	flagSet.Var(&args.exclude, "e", "Alias for -exclude")
+	flagSet.Var(&args.exclude, "exclude", "Exclude relative path from reverse view")
+
 	flagSet.IntVar(&args.notifypid, "notifypid", 0, "Send USR1 to the specified process after "+
 		"successful mount - used internally for daemonization")
 	flagSet.IntVar(&args.scryptn, "scryptn", configfile.ScryptDefaultLogN, "scrypt cost parameter logN. Possible values: 10-28. "+
 		"A lower value speeds up mounting and reduces its memory needs, but makes the password susceptible to brute-force attacks")
-	// Ignored otions
-	var dummyBool bool
-	ignoreText := "(ignored for compatibility)"
-	flagSet.BoolVar(&dummyBool, "rw", false, ignoreText)
-	flagSet.BoolVar(&dummyBool, "nosuid", false, ignoreText)
-	flagSet.BoolVar(&dummyBool, "nodev", false, ignoreText)
 	var dummyString string
 	flagSet.StringVar(&dummyString, "o", "", "For compatibility with mount(1), options can be also passed as a comma-separated list to -o on the end.")
 	// Actual parsing
 	err = flagSet.Parse(os.Args[1:])
 	if err == flag.ErrHelp {
+		helpShort()
 		os.Exit(0)
 	}
 	if err != nil {
-		tlog.Warn.Printf("You passed: %s", prettyArgs())
-		tlog.Fatal.Printf("%v", err)
+		tlog.Fatal.Printf("Invalid command line: %s. Try '%s -help'.", prettyArgs(), tlog.ProgramName)
 		os.Exit(exitcodes.Usage)
 	}
 	// "-openssl" needs some post-processing
@@ -210,13 +243,35 @@ func parseCliOpts() (args argContainer) {
 		tlog.Fatal.Printf("The options -extpass and -masterkey cannot be used at the same time")
 		os.Exit(exitcodes.Usage)
 	}
+	if args.extpass != "" && args.trezor {
+		tlog.Fatal.Printf("The options -extpass and -trezor cannot be used at the same time")
+		os.Exit(exitcodes.Usage)
+	}
 	return args
 }
 
 // prettyArgs pretty-prints the command-line arguments.
 func prettyArgs() string {
-	pa := fmt.Sprintf("%q", os.Args[1:])
+	pa := fmt.Sprintf("%v", os.Args)
 	// Get rid of "[" and "]"
 	pa = pa[1 : len(pa)-1]
 	return pa
 }
+
+// countOpFlags counts the number of operation flags we were passed.
+func countOpFlags(args *argContainer) int {
+	var count int
+	if args.info {
+		count++
+	}
+	if args.passwd {
+		count++
+	}
+	if args.init {
+		count++
+	}
+	if args.fsck {
+		count++
+	}
+	return count
+}
