@@ -45,6 +45,10 @@ type ConfFile struct {
 	// mounting. This mechanism is analogous to the ext4 feature flags that are
 	// stored in the superblock.
 	FeatureFlags []string
+	// TrezorPayload stores 32 random bytes used for unlocking the master key using
+	// a Trezor security module. The randomness makes sure that a unique unlock
+	// value is used for each gocryptfs filesystem.
+	TrezorPayload []byte `json:",omitempty"`
 	// Filename is the name of the config file. Not exported to JSON.
 	filename string
 }
@@ -64,10 +68,11 @@ func randBytesDevRandom(n int) []byte {
 	return b
 }
 
-// CreateConfFile - create a new config with a random key encrypted with
+// Create - create a new config with a random key encrypted with
 // "password" and write it to "filename".
 // Uses scrypt with cost parameter logN.
-func CreateConfFile(filename string, password string, plaintextNames bool, logN int, creator string, aessiv bool, devrandom bool) error {
+func Create(filename string, password []byte, plaintextNames bool,
+	logN int, creator string, aessiv bool, devrandom bool, trezorPayload []byte) error {
 	var cf ConfFile
 	cf.filename = filename
 	cf.Creator = creator
@@ -87,38 +92,45 @@ func CreateConfFile(filename string, password string, plaintextNames bool, logN
 	if aessiv {
 		cf.FeatureFlags = append(cf.FeatureFlags, knownFlags[FlagAESSIV])
 	}
-
-	// Generate new random master key
-	var key []byte
-	if devrandom {
-		key = randBytesDevRandom(cryptocore.KeyLen)
-	} else {
-		key = cryptocore.RandBytes(cryptocore.KeyLen)
+	if len(trezorPayload) > 0 {
+		cf.FeatureFlags = append(cf.FeatureFlags, knownFlags[FlagTrezor])
+		cf.TrezorPayload = trezorPayload
+	}
+	{
+		// Generate new random master key
+		var key []byte
+		if devrandom {
+			key = randBytesDevRandom(cryptocore.KeyLen)
+		} else {
+			key = cryptocore.RandBytes(cryptocore.KeyLen)
+		}
+		tlog.PrintMasterkeyReminder(key)
+		// Encrypt it using the password
+		// This sets ScryptObject and EncryptedKey
+		// Note: this looks at the FeatureFlags, so call it AFTER setting them.
+		cf.EncryptKey(key, password, logN)
+		for i := range key {
+			key[i] = 0
+		}
+		// key runs out of scope here
 	}
-
-	// Encrypt it using the password
-	// This sets ScryptObject and EncryptedKey
-	// Note: this looks at the FeatureFlags, so call it AFTER setting them.
-	cf.EncryptKey(key, password, logN)
-
 	// Write file to disk
 	return cf.WriteFile()
 }
 
-// LoadConfFile - read config file from disk and decrypt the
+// Load - read config file from disk and decrypt the
 // contained key using "password".
 // Returns the decrypted key and the ConfFile object
 //
 // If "password" is empty, the config file is read
 // but the key is not decrypted (returns nil in its place).
-func LoadConfFile(filename string, password string) ([]byte, *ConfFile, error) {
+func Load(filename string, password []byte) ([]byte, *ConfFile, error) {
 	var cf ConfFile
 	cf.filename = filename
 
 	// Read from disk
 	js, err := ioutil.ReadFile(filename)
 	if err != nil {
-		fmt.Printf("LoadConfFile: ReadFile: %#v\n", err)
 		return nil, nil, err
 	}
 
@@ -155,7 +167,7 @@ func LoadConfFile(filename string, password string) ([]byte, *ConfFile, error) {
 		}
 	}
 	if deprecatedFs {
-		fmt.Fprintf(os.Stderr, "\033[33m"+`
+		fmt.Fprintf(os.Stderr, tlog.ColorYellow+`
     The filesystem was created by gocryptfs v0.6 or earlier. This version of
     gocryptfs can no longer mount the filesystem.
     Please download gocryptfs v0.11 and upgrade your filesystem,
@@ -164,11 +176,11 @@ func LoadConfFile(filename string, password string) ([]byte, *ConfFile, error) {
     If you have trouble upgrading, join the discussion at
     https://github.com/rfjakob/gocryptfs/issues/29 .
 
-`+"\033[0m")
+`+tlog.ColorReset)
 
-		return nil, nil, fmt.Errorf("Deprecated filesystem")
+		return nil, nil, exitcodes.NewErr("Deprecated filesystem", exitcodes.DeprecatedFS)
 	}
-	if password == "" {
+	if len(password) == 0 {
 		// We have validated the config file, but without a password we cannot
 		// decrypt the master key. Return only the parsed config.
 		return nil, &cf, nil
@@ -196,15 +208,18 @@ func LoadConfFile(filename string, password string) ([]byte, *ConfFile, error) {
 // and store it in cf.EncryptedKey.
 // Uses scrypt with cost parameter logN and stores the scrypt parameters in
 // cf.ScryptObject.
-func (cf *ConfFile) EncryptKey(key []byte, password string, logN int) {
-	// Generate derived key from password
+func (cf *ConfFile) EncryptKey(key []byte, password []byte, logN int) {
+	// Generate scrypt-derived key from password
 	cf.ScryptObject = NewScryptKDF(logN)
 	scryptHash := cf.ScryptObject.DeriveKey(password)
-
 	// Lock master key using password-based key
 	useHKDF := cf.IsFeatureFlagSet(FlagHKDF)
 	ce := getKeyEncrypter(scryptHash, useHKDF)
 	cf.EncryptedKey = ce.EncryptBlock(key, 0, nil)
+	// Purge scrypt-derived key
+	for i := range scryptHash {
+		scryptHash[i] = 0
+	}
 }
 
 // WriteFile - write out config in JSON format to file "filename.tmp"
