@@ -2,7 +2,6 @@ package fusefrontend
 
 import (
 	"github.com/hanwen/go-fuse/fuse"
-	"github.com/rfjakob/gocryptfs/internal/cryptocore"
 )
 
 // Args is a container for arguments that are passed from main() to fusefrontend
@@ -10,7 +9,6 @@ type Args struct {
 	// Cipherdir is the backing storage directory (absolute path).
 	// For reverse mode, Cipherdir actually contains *plaintext* files.
 	Cipherdir      string
-	CryptoBackend  cryptocore.AEADTypeEnum
 	PlaintextNames bool
 	LongNames      bool
 	// Should we chown a file after it has been created?
@@ -26,17 +24,12 @@ type Args struct {
 	// location. If it is false, reverse mode maps ".gocryptfs.reverse.conf"
 	// to "gocryptfs.conf" in the plaintext dir.
 	ConfigCustom bool
-	// Raw64 is true when RawURLEncoding (without padding) should be used for
-	// file names.
-	// Corresponds to the Raw64 feature flag introduced in gocryptfs v1.2.
-	Raw64 bool
 	// NoPrealloc disables automatic preallocation before writing
 	NoPrealloc bool
-	// Use HKDF key derivation.
-	// Corresponds to the HKDF feature flag introduced in gocryptfs v1.3.
-	HKDF bool
 	// Try to serialize read operations, "-serialize_reads"
 	SerializeReads bool
 	// Force decode even if integrity check fails (openSSL only)
 	ForceDecode bool
+	// Exclude is a list of paths to make inaccessible
+	Exclude []string
 }
