@@ -17,7 +17,6 @@ import (
 	"github.com/hanwen/go-fuse/fuse/pathfs"
 
 	"github.com/rfjakob/gocryptfs/internal/contentenc"
-	"github.com/rfjakob/gocryptfs/internal/cryptocore"
 	"github.com/rfjakob/gocryptfs/internal/nametransform"
 	"github.com/rfjakob/gocryptfs/internal/serialize_reads"
 	"github.com/rfjakob/gocryptfs/internal/syscallcompat"
@@ -39,25 +38,32 @@ type FS struct {
 	// This lock is used by openWriteOnlyFile() to block concurrent opens while
 	// it relaxes the permissions on a file.
 	openWriteOnlyLock sync.RWMutex
+	// MitigatedCorruptions is used to report data corruption that is internally
+	// mitigated by ignoring the corrupt item. For example, when OpenDir() finds
+	// a corrupt filename, we still return the other valid filenames.
+	// The corruption is logged to syslog to inform the user,	and in addition,
+	// the corrupt filename is logged to this channel via
+	// reportMitigatedCorruption().
+	// "gocryptfs -fsck" reads from the channel to also catch these transparently-
+	// mitigated corruptions.
+	MitigatedCorruptions chan string
 }
 
 var _ pathfs.FileSystem = &FS{} // Verify that interface is implemented.
 
 // NewFS returns a new encrypted FUSE overlay filesystem.
-func NewFS(masterkey []byte, args Args) *FS {
-	cryptoCore := cryptocore.New(masterkey, args.CryptoBackend, contentenc.DefaultIVBits, args.HKDF, args.ForceDecode)
-	contentEnc := contentenc.New(cryptoCore, contentenc.DefaultBS, args.ForceDecode)
-	nameTransform := nametransform.New(cryptoCore.EMECipher, args.LongNames, args.Raw64)
-
+func NewFS(args Args, c *contentenc.ContentEnc, n *nametransform.NameTransform) *FS {
 	if args.SerializeReads {
 		serialize_reads.InitSerializer()
 	}
-
+	if len(args.Exclude) > 0 {
+		tlog.Warn.Printf("Forward mode does not support -exclude")
+	}
 	return &FS{
 		FileSystem:    pathfs.NewLoopbackFileSystem(args.Cipherdir),
 		args:          args,
-		nameTransform: nameTransform,
-		contentEnc:    contentEnc,
+		nameTransform: n,
+		contentEnc:    c,
 	}
 }
 
@@ -98,6 +104,11 @@ func (fs *FS) mangleOpenFlags(flags uint32) (newFlags int) {
 	}
 	// We also cannot open the file in append mode, we need to seek back for RMW
 	newFlags = newFlags &^ os.O_APPEND
+	// O_DIRECT accesses must be aligned in both offset and length. Due to our
+	// crypto header, alignment will be off, even if userspace makes aligned
+	// accesses. Running xfstests generic/013 on ext4 used to trigger lots of
+	// EINVAL errors due to missing alignment. Just fall back to buffered IO.
+	newFlags = newFlags &^ syscallcompat.O_DIRECT
 
 	return newFlags
 }
@@ -352,13 +363,32 @@ func (fs *FS) StatFs(path string) *fuse.StatfsOut {
 	return fs.FileSystem.StatFs(cPath)
 }
 
+// decryptSymlinkTarget: "cData64" is base64-decoded and decrypted
+// like file contents (GCM).
+// The empty string decrypts to the empty string.
+func (fs *FS) decryptSymlinkTarget(cData64 string) (string, error) {
+	if cData64 == "" {
+		return "", nil
+	}
+	cData, err := fs.nameTransform.B64.DecodeString(cData64)
+	if err != nil {
+		return "", err
+	}
+	data, err := fs.contentEnc.DecryptBlock([]byte(cData), 0, nil)
+	if err != nil {
+		return "", err
+	}
+	return string(data), nil
+}
+
 // Readlink implements pathfs.Filesystem.
-func (fs *FS) Readlink(path string, context *fuse.Context) (out string, status fuse.Status) {
-	cPath, err := fs.getBackingPath(path)
+func (fs *FS) Readlink(relPath string, context *fuse.Context) (out string, status fuse.Status) {
+	cPath, err := fs.encryptPath(relPath)
 	if err != nil {
 		return "", fuse.ToStatus(err)
 	}
-	cTarget, err := os.Readlink(cPath)
+	cAbsPath := filepath.Join(fs.args.Cipherdir, cPath)
+	cTarget, err := os.Readlink(cAbsPath)
 	if err != nil {
 		return "", fuse.ToStatus(err)
 	}
@@ -366,14 +396,9 @@ func (fs *FS) Readlink(path string, context *fuse.Context) (out string, status f
 		return cTarget, fuse.OK
 	}
 	// Symlinks are encrypted like file contents (GCM) and base64-encoded
-	cBinTarget, err := fs.nameTransform.B64.DecodeString(cTarget)
+	target, err := fs.decryptSymlinkTarget(cTarget)
 	if err != nil {
-		tlog.Warn.Printf("Readlink: %v", err)
-		return "", fuse.EIO
-	}
-	target, err := fs.contentEnc.DecryptBlock([]byte(cBinTarget), 0, nil)
-	if err != nil {
-		tlog.Warn.Printf("Readlink: %v", err)
+		tlog.Warn.Printf("Readlink %q: decrypting target failed: %v", cPath, err)
 		return "", fuse.EIO
 	}
 	return string(target), fuse.OK
@@ -404,6 +429,18 @@ func (fs *FS) Unlink(path string, context *fuse.Context) (code fuse.Status) {
 	return fuse.ToStatus(err)
 }
 
+// encryptSymlinkTarget: "data" is encrypted like file contents (GCM)
+// and base64-encoded.
+// The empty string encrypts to the empty string.
+func (fs *FS) encryptSymlinkTarget(data string) (cData64 string) {
+	if data == "" {
+		return ""
+	}
+	cData := fs.contentEnc.EncryptBlock([]byte(data), 0, nil)
+	cData64 = fs.nameTransform.B64.EncodeToString(cData)
+	return cData64
+}
+
 // Symlink implements pathfs.Filesystem.
 func (fs *FS) Symlink(target string, linkName string, context *fuse.Context) (code fuse.Status) {
 	tlog.Debug.Printf("Symlink(\"%s\", \"%s\")", target, linkName)
@@ -415,11 +452,10 @@ func (fs *FS) Symlink(target string, linkName string, context *fuse.Context) (co
 		return fuse.ToStatus(err)
 	}
 	defer dirfd.Close()
-	var cTarget string = target
+	cTarget := target
 	if !fs.args.PlaintextNames {
 		// Symlinks are encrypted like file contents (GCM) and base64-encoded
-		cBinTarget := fs.contentEnc.EncryptBlock([]byte(target), 0, nil)
-		cTarget = fs.nameTransform.B64.EncodeToString(cBinTarget)
+		cTarget = fs.encryptSymlinkTarget(target)
 	}
 	// Create ".name" file to store long file name (except in PlaintextNames mode)
 	if !fs.args.PlaintextNames && nametransform.IsLongContent(cName) {
@@ -583,22 +619,19 @@ func (fs *FS) Access(path string, mode uint32, context *fuse.Context) (code fuse
 	return fuse.ToStatus(syscall.Access(cPath, mode))
 }
 
-// GetXAttr implements pathfs.Filesystem.
-func (fs *FS) GetXAttr(name string, attr string, context *fuse.Context) ([]byte, fuse.Status) {
-	return nil, fuse.ENOSYS
-}
-
-// SetXAttr implements pathfs.Filesystem.
-func (fs *FS) SetXAttr(name string, attr string, data []byte, flags int, context *fuse.Context) fuse.Status {
-	return fuse.ENOSYS
-}
-
-// ListXAttr implements pathfs.Filesystem.
-func (fs *FS) ListXAttr(name string, context *fuse.Context) ([]string, fuse.Status) {
-	return nil, fuse.ENOSYS
-}
-
-// RemoveXAttr implements pathfs.Filesystem.
-func (fs *FS) RemoveXAttr(name string, attr string, context *fuse.Context) fuse.Status {
-	return fuse.ENOSYS
+// reportMitigatedCorruption is used to report a corruption that was transparently
+// mitigated and did not return an error to the user. Pass the name of the corrupt
+// item (filename for OpenDir(), xattr name for ListXAttr() etc).
+// See the MitigatedCorruptions channel for more info.
+func (fs *FS) reportMitigatedCorruption(item string) {
+	if fs.MitigatedCorruptions == nil {
+		return
+	}
+	select {
+	case fs.MitigatedCorruptions <- item:
+	case <-time.After(1 * time.Second):
+		tlog.Warn.Printf("BUG: reportCorruptItem: timeout")
+		//debug.PrintStack()
+		return
+	}
 }
