@@ -8,7 +8,6 @@ import (
 	"os"
 	"path/filepath"
 	"runtime"
-	"sync"
 	"syscall"
 
 	"golang.org/x/sys/unix"
@@ -257,9 +256,6 @@ retry:
 	return fuse.OK
 }
 
-// If syscallcompat.HaveGetdents is false we will warn once about it
-var haveGetdentsWarnOnce sync.Once
-
 // OpenDir implements pathfs.FileSystem
 func (fs *FS) OpenDir(dirName string, context *fuse.Context) ([]fuse.DirEntry, fuse.Status) {
 	tlog.Debug.Printf("OpenDir(%s)", dirName)
@@ -290,12 +286,13 @@ func (fs *FS) OpenDir(dirName string, context *fuse.Context) ([]fuse.DirEntry, f
 			cachedIV, err = nametransform.ReadDirIV(cDirAbsPath)
 			if err != nil {
 				fs.dirIVLock.RUnlock()
-				// This can happen during normal operation when the directory has
-				// been deleted concurrently. But it can also mean that the
-				// gocryptfs.diriv is missing due to an error, so log the event
-				// at "info" level.
-				tlog.Info.Printf("OpenDir: %v", err)
-				return nil, fuse.ToStatus(err)
+				// The directory itself does not exist
+				if err == syscall.ENOENT {
+					return nil, fuse.ENOENT
+				}
+				// Any other problem warrants an error message
+				tlog.Warn.Printf("OpenDir %q: could not read gocryptfs.diriv: %v", cDirName, err)
+				return nil, fuse.EIO
 			}
 			fs.nameTransform.DirIVCache.Store(dirName, cachedIV, cDirName)
 			fs.dirIVLock.RUnlock()
@@ -329,6 +326,7 @@ func (fs *FS) OpenDir(dirName string, context *fuse.Context) ([]fuse.DirEntry, f
 			if err != nil {
 				tlog.Warn.Printf("OpenDir %q: invalid entry %q: Could not read .name: %v",
 					cDirName, cName, err)
+				fs.reportMitigatedCorruption(cName)
 				errorCount++
 				continue
 			}
@@ -341,6 +339,7 @@ func (fs *FS) OpenDir(dirName string, context *fuse.Context) ([]fuse.DirEntry, f
 		if err != nil {
 			tlog.Warn.Printf("OpenDir %q: invalid entry %q: %v",
 				cDirName, cName, err)
+			fs.reportMitigatedCorruption(cName)
 			if runtime.GOOS == "darwin" && cName == dsStoreName {
 				// MacOS creates lots of these files. Log the warning but don't
 				// increment errorCount - does not warrant returning EIO.
