@@ -3,6 +3,7 @@ package syscallcompat
 import (
 	"bytes"
 	"os"
+	"runtime"
 	"syscall"
 	"testing"
 )
@@ -12,6 +13,11 @@ func TestReadlinkat(t *testing.T) {
 		target := string(bytes.Repeat([]byte("x"), targetLen))
 		err := os.Symlink(target, tmpDir+"/readlinkat")
 		if err != nil {
+			if targetLen > 1000 && runtime.GOOS == "darwin" {
+				// Symlinks longer than 1024 (?) bytes are not supported on
+				// MacOS
+				continue
+			}
 			t.Fatal(err)
 		}
 		target2, err := Readlinkat(tmpDirFd, "readlinkat")
