@@ -2,6 +2,7 @@ package main
 
 import (
 	"fmt"
+	"log"
 	"os"
 	"path/filepath"
 	"runtime"
@@ -33,23 +34,33 @@ var raceDetector bool
 
 // loadConfig loads the config file "args.config", prompting the user for the password
 func loadConfig(args *argContainer) (masterkey []byte, confFile *configfile.ConfFile, err error) {
-	// Check if the file can be opened at all before prompting for a password
-	fd, err := os.Open(args.config)
+	// First check if the file can be read at all, and find out if a Trezor should
+	// be used instead of a password.
+	_, cf1, err := configfile.Load(args.config, nil)
 	if err != nil {
 		tlog.Fatal.Printf("Cannot open config file: %v", err)
-		return nil, nil, exitcodes.NewErr(err.Error(), exitcodes.OpenConf)
+		return nil, nil, err
 	}
-	fd.Close()
-	// The user has passed the master key (probably because he forgot the
-	// password).
+	// The user has passed the master key on the command line (probably because
+	// he forgot the password).
 	if args.masterkey != "" {
-		masterkey = parseMasterKey(args.masterkey)
-		_, confFile, err = configfile.LoadConfFile(args.config, "")
-	} else {
-		pw := readpassword.Once(args.extpass)
-		tlog.Info.Println("Decrypting master key")
-		masterkey, confFile, err = configfile.LoadConfFile(args.config, pw)
+		masterkey = parseMasterKey(args.masterkey, false)
+		return masterkey, cf1, nil
 	}
+	var pw []byte
+	if cf1.IsFeatureFlagSet(configfile.FlagTrezor) {
+		// Get binary data from from Trezor
+		pw = readpassword.Trezor(cf1.TrezorPayload)
+	} else {
+		// Normal password entry
+		pw = readpassword.Once(args.extpass, "")
+	}
+	tlog.Info.Println("Decrypting master key")
+	masterkey, confFile, err = configfile.Load(args.config, pw)
+	for i := range pw {
+		pw[i] = 0
+	}
+
 	if err != nil {
 		tlog.Fatal.Println(err)
 		return nil, nil, err
@@ -58,15 +69,43 @@ func loadConfig(args *argContainer) (masterkey []byte, confFile *configfile.Conf
 }
 
 // changePassword - change the password of config file "filename"
+// Does not return (calls os.Exit both on success and on error).
 func changePassword(args *argContainer) {
-	masterkey, confFile, err := loadConfig(args)
+	// Parse the config file, but do not unlock the master key. We only want to
+	// know if the Trezor flag is set.
+	_, cf1, err := configfile.Load(args.config, nil)
 	if err != nil {
-		exitcodes.Exit(err)
+		tlog.Fatal.Printf("Cannot open config file: %v", err)
+		os.Exit(exitcodes.LoadConf)
 	}
-	tlog.Info.Println("Please enter your new password.")
-	newPw := readpassword.Twice(args.extpass)
-	readpassword.CheckTrailingGarbage()
-	confFile.EncryptKey(masterkey, newPw, confFile.ScryptObject.LogN())
+	if cf1.IsFeatureFlagSet(configfile.FlagTrezor) {
+		tlog.Fatal.Printf("Password change is not supported on Trezor-enabled filesystems.")
+		os.Exit(exitcodes.Usage)
+	}
+	var confFile *configfile.ConfFile
+	{
+		var masterkey []byte
+		masterkey, confFile, err = loadConfig(args)
+		if err != nil {
+			exitcodes.Exit(err)
+		}
+		if len(masterkey) == 0 {
+			log.Panic("empty masterkey")
+		}
+		tlog.Info.Println("Please enter your new password.")
+		newPw := readpassword.Twice(args.extpass)
+		readpassword.CheckTrailingGarbage()
+		confFile.EncryptKey(masterkey, newPw, confFile.ScryptObject.LogN())
+		for i := range newPw {
+			newPw[i] = 0
+		}
+		for i := range masterkey {
+			masterkey[i] = 0
+		}
+		// masterkey and newPw run out of scope here
+	}
+	// Are we resetting the password without knowing the old one using
+	// "-masterkey"?
 	if args.masterkey != "" {
 		bak := args.config + ".bak"
 		err = os.Link(args.config, bak)
@@ -85,22 +124,28 @@ func changePassword(args *argContainer) {
 		os.Exit(exitcodes.WriteConf)
 	}
 	tlog.Info.Printf(tlog.ColorGreen + "Password changed." + tlog.ColorReset)
-	os.Exit(0)
 }
 
 // printVersion prints a version string like this:
 // gocryptfs v0.12-36-ge021b9d-dirty; go-fuse a4c968c; 2016-07-03 go1.6.2
 func printVersion() {
-	buildFlags := ""
+	var tagsSlice []string
 	if stupidgcm.BuiltWithoutOpenssl {
-		buildFlags = " without_openssl"
+		tagsSlice = append(tagsSlice, "without_openssl")
+	}
+	if readpassword.TrezorSupport {
+		tagsSlice = append(tagsSlice, "enable_trezor")
+	}
+	tags := ""
+	if tagsSlice != nil {
+		tags = " " + strings.Join(tagsSlice, " ")
 	}
 	built := fmt.Sprintf("%s %s", BuildDate, runtime.Version())
 	if raceDetector {
 		built += " -race"
 	}
 	fmt.Printf("%s %s%s; go-fuse %s; %s\n",
-		tlog.ProgramName, GitVersion, buildFlags, GitVersionFuse, built)
+		tlog.ProgramName, GitVersion, tags, GitVersionFuse, built)
 }
 
 func main() {
@@ -109,6 +154,11 @@ func main() {
 		// On a 2-core machine, setting maxprocs to 4 gives 10% better performance
 		runtime.GOMAXPROCS(4)
 	}
+	// mount(1) unsets PATH. Since exec.Command does not handle this case, we set
+	// PATH to a default value if it's empty or unset.
+	if os.Getenv("PATH") == "" {
+		os.Setenv("PATH", "/usr/sbin:/usr/bin:/sbin:/bin")
+	}
 	var err error
 	// Parse all command-line options (i.e. arguments starting with "-")
 	// into "args". Path arguments are parsed below.
@@ -141,7 +191,7 @@ func main() {
 	}
 	if args.wpanic {
 		tlog.Warn.Wpanic = true
-		tlog.Debug.Printf("Panicing on warnings")
+		tlog.Debug.Printf("Panicking on warnings")
 	}
 	// Every operation below requires CIPHERDIR. Exit if we don't have it.
 	if flagSet.NArg() == 0 {
@@ -157,7 +207,7 @@ func main() {
 	}
 	// Check that CIPHERDIR exists
 	args.cipherdir, _ = filepath.Abs(flagSet.Arg(0))
-	err = checkDir(args.cipherdir)
+	err = isDir(args.cipherdir)
 	if err != nil {
 		tlog.Fatal.Printf("Invalid cipherdir: %v", err)
 		os.Exit(exitcodes.CipherDir)
@@ -169,6 +219,11 @@ func main() {
 	// "-reverse" implies "-aessiv"
 	if args.reverse {
 		args.aessiv = true
+	} else {
+		if args.exclude != nil {
+			tlog.Fatal.Printf("-exclude only works in reverse mode")
+			os.Exit(exitcodes.ExcludeError)
+		}
 	}
 	// "-config"
 	if args.config != "" {
@@ -229,46 +284,47 @@ func main() {
 		tlog.Debug.Printf("OpenSSL enabled")
 	}
 	// Operation flags
-	if args.info && args.init || args.info && args.passwd || args.passwd && args.init {
-		tlog.Fatal.Printf("At most one of -info, -init, -passwd is allowed")
+	nOps := countOpFlags(&args)
+	if nOps == 0 {
+		// Default operation: mount.
+		if flagSet.NArg() != 2 {
+			prettyArgs := prettyArgs()
+			tlog.Info.Printf("Wrong number of arguments (have %d, want 2). You passed: %s",
+				flagSet.NArg(), prettyArgs)
+			tlog.Fatal.Printf("Usage: %s [OPTIONS] CIPHERDIR MOUNTPOINT [-o COMMA-SEPARATED-OPTIONS]", tlog.ProgramName)
+			os.Exit(exitcodes.Usage)
+		}
+		doMount(&args)
+		// Don't call os.Exit to give deferred functions a chance to run
+		return
+	}
+	if nOps > 1 {
+		tlog.Fatal.Printf("At most one of -info, -init, -passwd, -fsck is allowed")
+		os.Exit(exitcodes.Usage)
+	}
+	if flagSet.NArg() != 1 {
+		tlog.Fatal.Printf("The options -info, -init, -passwd, -fsck take exactly one argument, %d given",
+			flagSet.NArg())
 		os.Exit(exitcodes.Usage)
 	}
 	// "-info"
 	if args.info {
-		if flagSet.NArg() > 1 {
-			tlog.Fatal.Printf("Usage: %s -info CIPHERDIR", tlog.ProgramName)
-			os.Exit(exitcodes.Usage)
-		}
-		info(args.config) // does not return
+		info(args.config)
+		os.Exit(0)
 	}
 	// "-init"
 	if args.init {
-		if flagSet.NArg() > 1 {
-			tlog.Fatal.Printf("Usage: %s -init [OPTIONS] CIPHERDIR", tlog.ProgramName)
-			os.Exit(exitcodes.Usage)
-		}
-		initDir(&args) // does not return
+		initDir(&args)
+		os.Exit(0)
 	}
 	// "-passwd"
 	if args.passwd {
-		if flagSet.NArg() > 1 {
-			tlog.Fatal.Printf("Usage: %s -passwd [OPTIONS] CIPHERDIR", tlog.ProgramName)
-			os.Exit(exitcodes.Usage)
-		}
-		changePassword(&args) // does not return
+		changePassword(&args)
+		os.Exit(0)
 	}
-	// Default operation: mount.
-	if flagSet.NArg() != 2 {
-		prettyArgs := prettyArgs()
-		tlog.Info.Printf("Wrong number of arguments (have %d, want 2). You passed: %s",
-			flagSet.NArg(), prettyArgs)
-		tlog.Fatal.Printf("Usage: %s [OPTIONS] CIPHERDIR MOUNTPOINT [-o COMMA-SEPARATED-OPTIONS]", tlog.ProgramName)
-		os.Exit(exitcodes.Usage)
+	// "-fsck"
+	if args.fsck {
+		fsck(&args)
+		os.Exit(0)
 	}
-	ret := doMount(&args)
-	if ret != 0 {
-		os.Exit(ret)
-	}
-	// Don't call os.Exit on success to give deferred functions a chance to
-	// run
 }
