@@ -3,6 +3,7 @@ package main
 import (
 	"encoding/json"
 	"fmt"
+	"log"
 	"log/syslog"
 	"net"
 	"os"
@@ -21,18 +22,19 @@ import (
 	"github.com/hanwen/go-fuse/fuse/pathfs"
 
 	"github.com/rfjakob/gocryptfs/internal/configfile"
+	"github.com/rfjakob/gocryptfs/internal/contentenc"
 	"github.com/rfjakob/gocryptfs/internal/cryptocore"
 	"github.com/rfjakob/gocryptfs/internal/ctlsock"
 	"github.com/rfjakob/gocryptfs/internal/exitcodes"
 	"github.com/rfjakob/gocryptfs/internal/fusefrontend"
 	"github.com/rfjakob/gocryptfs/internal/fusefrontend_reverse"
-	"github.com/rfjakob/gocryptfs/internal/readpassword"
+	"github.com/rfjakob/gocryptfs/internal/nametransform"
 	"github.com/rfjakob/gocryptfs/internal/tlog"
 )
 
 // doMount mounts an encrypted directory.
 // Called from main.
-func doMount(args *argContainer) int {
+func doMount(args *argContainer) {
 	// Check mountpoint
 	var err error
 	args.mountpoint, err = filepath.Abs(flagSet.Arg(1))
@@ -55,9 +57,9 @@ func doMount(args *argContainer) int {
 		os.Exit(exitcodes.MountPoint)
 	}
 	if args.nonempty {
-		err = checkDir(args.mountpoint)
+		err = isDir(args.mountpoint)
 	} else {
-		err = checkDirEmpty(args.mountpoint)
+		err = isDirEmpty(args.mountpoint)
 		// OSXFuse will create the mountpoint for us ( https://github.com/rfjakob/gocryptfs/issues/194 )
 		if runtime.GOOS == "darwin" && os.IsNotExist(err) {
 			tlog.Info.Printf("Mountpoint %q does not exist, but should be created by OSXFuse",
@@ -86,41 +88,19 @@ func doMount(args *argContainer) int {
 		defer func() {
 			err = sock.Close()
 			if err != nil {
-				tlog.Warn.Print(err)
+				tlog.Warn.Printf("ctlsock close: %v", err)
 			}
 		}()
 	}
-	// Get master key (may prompt for the password)
-	var masterkey []byte
-	var confFile *configfile.ConfFile
-	if args.masterkey != "" {
-		// "-masterkey"
-		masterkey = parseMasterKey(args.masterkey)
-	} else if args.zerokey {
-		// "-zerokey"
-		tlog.Info.Printf("Using all-zero dummy master key.")
-		tlog.Info.Printf(tlog.ColorYellow +
-			"ZEROKEY MODE PROVIDES NO SECURITY AT ALL AND SHOULD ONLY BE USED FOR TESTING." +
-			tlog.ColorReset)
-		masterkey = make([]byte, cryptocore.KeyLen)
-	} else {
-		// Load master key from config file
-		// Prompts the user for the password
-		masterkey, confFile, err = loadConfig(args)
-		if err != nil {
-			if args._ctlsockFd != nil {
-				// Close the socket file (which also deletes it)
-				args._ctlsockFd.Close()
-			}
-			exitcodes.Exit(err)
-		}
-		readpassword.CheckTrailingGarbage()
-		printMasterKey(masterkey)
-	}
 	// We cannot use JSON for pretty-printing as the fields are unexported
 	tlog.Debug.Printf("cli args: %#v", args)
-	// Initialize FUSE server
-	srv := initFuseFrontend(masterkey, args, confFile)
+	// Initialize gocryptfs
+	fs, wipeKeys := initFuseFrontend(args)
+	// Initialize go-fuse FUSE server
+	srv := initGoFuse(fs, args)
+	// Try to wipe secrect keys from memory after unmount
+	defer wipeKeys()
+
 	tlog.Info.Println(tlog.ColorGreen + "Filesystem mounted and ready." + tlog.ColorReset)
 	// We have been forked into the background, as evidenced by the set
 	// "notifypid".
@@ -159,7 +139,6 @@ func doMount(args *argContainer) int {
 	debug.FreeOSMemory()
 	// Jump into server loop. Returns when it gets an umount request from the kernel.
 	srv.Serve()
-	return 0
 }
 
 // setOpenFileLimit tries to increase the open file limit to 4096 (the default hard
@@ -182,9 +161,18 @@ func setOpenFileLimit() {
 	}
 }
 
+// ctlsockFs satisfies both the pathfs.FileSystem and the ctlsock.Interface
+// interfaces
+type ctlsockFs interface {
+	pathfs.FileSystem
+	ctlsock.Interface
+}
+
 // initFuseFrontend - initialize gocryptfs/fusefrontend
 // Calls os.Exit on errors
-func initFuseFrontend(masterkey []byte, args *argContainer, confFile *configfile.ConfFile) *fuse.Server {
+func initFuseFrontend(args *argContainer) (pfs pathfs.FileSystem, wipeKeys func()) {
+	// Get master key (may prompt for the password) and read config file
+	masterkey, confFile := getMasterKey(args)
 	// Reconciliate CLI and config file arguments into a fusefrontend.Args struct
 	// that is passed to the filesystem implementation
 	cryptoBackend := cryptocore.BackendGoGCM
@@ -203,23 +191,21 @@ func initFuseFrontend(masterkey []byte, args *argContainer, confFile *configfile
 		Cipherdir:      args.cipherdir,
 		PlaintextNames: args.plaintextnames,
 		LongNames:      args.longnames,
-		CryptoBackend:  cryptoBackend,
 		ConfigCustom:   args._configCustom,
-		Raw64:          args.raw64,
 		NoPrealloc:     args.noprealloc,
-		HKDF:           args.hkdf,
 		SerializeReads: args.serialize_reads,
 		ForceDecode:    args.forcedecode,
 		ForceOwner:     args._forceOwner,
+		Exclude:        args.exclude,
 	}
 	// confFile is nil when "-zerokey" or "-masterkey" was used
 	if confFile != nil {
 		// Settings from the config file override command line args
 		frontendArgs.PlaintextNames = confFile.IsFeatureFlagSet(configfile.FlagPlaintextNames)
-		frontendArgs.Raw64 = confFile.IsFeatureFlagSet(configfile.FlagRaw64)
-		frontendArgs.HKDF = confFile.IsFeatureFlagSet(configfile.FlagHKDF)
+		args.raw64 = confFile.IsFeatureFlagSet(configfile.FlagRaw64)
+		args.hkdf = confFile.IsFeatureFlagSet(configfile.FlagHKDF)
 		if confFile.IsFeatureFlagSet(configfile.FlagAESSIV) {
-			frontendArgs.CryptoBackend = cryptocore.BackendAESSIV
+			cryptoBackend = cryptocore.BackendAESSIV
 		} else if args.reverse {
 			tlog.Fatal.Printf("AES-SIV is required by reverse mode, but not enabled in the config file")
 			os.Exit(exitcodes.Usage)
@@ -232,8 +218,37 @@ func initFuseFrontend(masterkey []byte, args *argContainer, confFile *configfile
 	}
 	jsonBytes, _ := json.MarshalIndent(frontendArgs, "", "\t")
 	tlog.Debug.Printf("frontendArgs: %s", string(jsonBytes))
-	var finalFs pathfs.FileSystem
-	var ctlSockBackend ctlsock.Interface
+
+	// Init crypto backend
+	cCore := cryptocore.New(masterkey, cryptoBackend, contentenc.DefaultIVBits, args.hkdf, args.forcedecode)
+	cEnc := contentenc.New(cCore, contentenc.DefaultBS, args.forcedecode)
+	nameTransform := nametransform.New(cCore.EMECipher, frontendArgs.LongNames, args.raw64)
+	// After the crypto backend is initialized,
+	// we can purge the master key from memory.
+	for i := range masterkey {
+		masterkey[i] = 0
+	}
+	masterkey = nil
+	// Spawn fusefrontend
+	var fs ctlsockFs
+	if args.reverse {
+		if cryptoBackend != cryptocore.BackendAESSIV {
+			log.Panic("reverse mode must use AES-SIV, everything else is insecure")
+		}
+		fs = fusefrontend_reverse.NewFS(frontendArgs, cEnc, nameTransform)
+
+	} else {
+		fs = fusefrontend.NewFS(frontendArgs, cEnc, nameTransform)
+	}
+	// We have opened the socket early so that we cannot fail here after
+	// asking the user for the password
+	if args._ctlsockFd != nil {
+		go ctlsock.Serve(args._ctlsockFd, fs)
+	}
+	return fs, func() { cCore.Wipe() }
+}
+
+func initGoFuse(fs pathfs.FileSystem, args *argContainer) *fuse.Server {
 	// pathFsOpts are passed into go-fuse/pathfs
 	pathFsOpts := &pathfs.PathNodeFsOptions{ClientInodes: true}
 	if args.sharedstorage {
@@ -243,32 +258,12 @@ func initFuseFrontend(masterkey []byte, args *argContainer, confFile *configfile
 		pathFsOpts.ClientInodes = false
 	}
 	if args.reverse {
-		// The dance with the intermediate variables is because we need to
-		// cast the FS into pathfs.FileSystem *and* ctlsock.Interface. This
-		// avoids using interface{}.
-		fs := fusefrontend_reverse.NewFS(masterkey, frontendArgs)
-		finalFs = fs
-		ctlSockBackend = fs
 		// Reverse mode is read-only, so we don't need a working link().
 		// Disable hard link tracking to avoid strange breakage on duplicate
 		// inode numbers ( https://github.com/rfjakob/gocryptfs/issues/149 ).
 		pathFsOpts.ClientInodes = false
-	} else {
-		fs := fusefrontend.NewFS(masterkey, frontendArgs)
-		finalFs = fs
-		ctlSockBackend = fs
 	}
-	// fusefrontend / fusefrontend_reverse have initialized their crypto with
-	// derived keys (HKDF), we can purge the master key from memory.
-	for i := range masterkey {
-		masterkey[i] = 0
-	}
-	// We have opened the socket early so that we cannot fail here after
-	// asking the user for the password
-	if args._ctlsockFd != nil {
-		go ctlsock.Serve(args._ctlsockFd, ctlSockBackend)
-	}
-	pathFs := pathfs.NewPathNodeFs(finalFs, pathFsOpts)
+	pathFs := pathfs.NewPathNodeFs(fs, pathFsOpts)
 	var fuseOpts *nodefs.Options
 	if args.sharedstorage {
 		// sharedstorage mode sets all cache timeouts to zero so changes to the
@@ -290,7 +285,7 @@ func initFuseFrontend(masterkey []byte, args *argContainer, confFile *configfile
 		// sync.Pool buffer pools are sized acc. to the default. Users may set
 		// the kernel constant higher, and Synology NAS kernels are known to
 		// have it >128kiB. We cannot handle more than 128kiB, so we tell
-		// the kernel to limit the size explicitely.
+		// the kernel to limit the size explicitly.
 		MaxWrite: fuse.MAX_KERNEL_WRITE,
 		Options:  []string{fmt.Sprintf("max_read=%d", fuse.MAX_KERNEL_WRITE)},
 	}
@@ -320,17 +315,33 @@ func initFuseFrontend(masterkey []byte, args *argContainer, confFile *configfile
 	if args.reverse {
 		mOpts.Name += "-reverse"
 	}
-
 	// Add a volume name if running osxfuse. Otherwise the Finder will show it as
 	// something like "osxfuse Volume 0 (gocryptfs)".
 	if runtime.GOOS == "darwin" {
 		mOpts.Options = append(mOpts.Options, "volname="+path.Base(args.mountpoint))
 	}
-
 	// The kernel enforces read-only operation, we just have to pass "ro".
 	// Reverse mounts are always read-only.
 	if args.ro || args.reverse {
 		mOpts.Options = append(mOpts.Options, "ro")
+	} else if args.rw {
+		mOpts.Options = append(mOpts.Options, "rw")
+	}
+	// If both "nosuid" and "suid" were passed, the safer option wins.
+	if args.nosuid {
+		mOpts.Options = append(mOpts.Options, "nosuid")
+	} else if args.suid {
+		mOpts.Options = append(mOpts.Options, "suid")
+	}
+	if args.nodev {
+		mOpts.Options = append(mOpts.Options, "nodev")
+	} else if args.dev {
+		mOpts.Options = append(mOpts.Options, "dev")
+	}
+	if args.noexec {
+		mOpts.Options = append(mOpts.Options, "noexec")
+	} else if args.exec {
+		mOpts.Options = append(mOpts.Options, "exec")
 	}
 	// Add additional mount options (if any) after the stock ones, so the user has
 	// a chance to override them.
@@ -341,7 +352,7 @@ func initFuseFrontend(masterkey []byte, args *argContainer, confFile *configfile
 	}
 	srv, err := fuse.NewServer(conn.RawFS(), args.mountpoint, &mOpts)
 	if err != nil {
-		tlog.Fatal.Printf("fuse.NewServer failed: %v", err)
+		tlog.Fatal.Printf("fuse.NewServer failed: %s", strings.TrimSpace(err.Error()))
 		if runtime.GOOS == "darwin" {
 			tlog.Info.Printf("Maybe you should run: /Library/Filesystems/osxfuse.fs/Contents/Resources/load_osxfuse")
 		}
@@ -365,7 +376,7 @@ func handleSigint(srv *fuse.Server, mountpoint string) {
 		<-ch
 		err := srv.Unmount()
 		if err != nil {
-			tlog.Warn.Print(err)
+			tlog.Warn.Printf("handleSigint: srv.Unmount returned %v", err)
 			if runtime.GOOS == "linux" {
 				// MacOSX does not support lazy unmount
 				tlog.Info.Printf("Trying lazy unmount")
