@@ -3,6 +3,7 @@ package cli
 // Test CLI operations like "-init", "-password" etc
 
 import (
+	"fmt"
 	"io/ioutil"
 	"os"
 	"os/exec"
@@ -16,16 +17,24 @@ import (
 	"github.com/rfjakob/gocryptfs/tests/test_helpers"
 )
 
+var testPw = []byte("test")
+
 func TestMain(m *testing.M) {
 	test_helpers.ResetTmpDir(false)
+	before := test_helpers.ListFds()
 	r := m.Run()
+	after := test_helpers.ListFds()
+	if len(before) != len(after) {
+		fmt.Printf("fd leak? before, after:\n%v\n%v\n", before, after)
+		os.Exit(1)
+	}
 	os.Exit(r)
 }
 
 // Test -init flag
 func TestInit(t *testing.T) {
 	dir := test_helpers.InitFS(t)
-	_, c, err := configfile.LoadConfFile(dir+"/"+configfile.ConfDefaultName, "test")
+	_, c, err := configfile.Load(dir+"/"+configfile.ConfDefaultName, testPw)
 	if err != nil {
 		t.Fatal(err)
 	}
@@ -42,7 +51,7 @@ func TestInitDevRandom(t *testing.T) {
 // Test -init with -aessiv
 func TestInitAessiv(t *testing.T) {
 	dir := test_helpers.InitFS(t, "-aessiv")
-	_, c, err := configfile.LoadConfFile(dir+"/"+configfile.ConfDefaultName, "test")
+	_, c, err := configfile.Load(dir+"/"+configfile.ConfDefaultName, testPw)
 	if err != nil {
 		t.Fatal(err)
 	}
@@ -54,7 +63,7 @@ func TestInitAessiv(t *testing.T) {
 // Test -init with -reverse
 func TestInitReverse(t *testing.T) {
 	dir := test_helpers.InitFS(t, "-reverse")
-	_, c, err := configfile.LoadConfFile(dir+"/"+configfile.ConfReverseName, "test")
+	_, c, err := configfile.Load(dir+"/"+configfile.ConfReverseName, testPw)
 	if err != nil {
 		t.Fatal(err)
 	}
@@ -330,8 +339,7 @@ func TestMountPasswordIncorrect(t *testing.T) {
 	cDir := test_helpers.InitFS(t) // Create filesystem with password "test"
 	pDir := cDir + ".mnt"
 	err := test_helpers.Mount(cDir, pDir, false, "-extpass", "echo WRONG", "-wpanic=false")
-	//          vvvvvvvvvvvvvv OMG vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv
-	exitCode := err.(*exec.ExitError).Sys().(syscall.WaitStatus).ExitStatus()
+	exitCode := test_helpers.ExtractCmdExitCode(err)
 	if exitCode != exitcodes.PasswordIncorrect {
 		t.Errorf("want=%d, got=%d", exitcodes.PasswordIncorrect, exitCode)
 	}
@@ -360,7 +368,7 @@ func TestPasswdPasswordIncorrect(t *testing.T) {
 		t.Fatal(err)
 	}
 	err = cmd.Wait()
-	exitCode := err.(*exec.ExitError).Sys().(syscall.WaitStatus).ExitStatus()
+	exitCode := test_helpers.ExtractCmdExitCode(err)
 	if exitCode != exitcodes.PasswordIncorrect {
 		t.Errorf("want=%d, got=%d", exitcodes.PasswordIncorrect, exitCode)
 	}
@@ -403,6 +411,7 @@ func TestMountBackground(t *testing.T) {
 			// We should get io.EOF when the child closes stdout
 			// and stderr.
 			if err != nil {
+				pr.Close()
 				c1 <- struct{}{}
 				return
 			}
@@ -415,3 +424,74 @@ func TestMountBackground(t *testing.T) {
 		t.Fatal("timeout")
 	}
 }
+
+// Test that "gocryptfs -init -info CIPHERDIR" returns an error to the
+// user. Only one operation flag is allowed.
+func TestMultipleOperationFlags(t *testing.T) {
+	// Test all combinations
+	opFlags := []string{"-init", "-info", "-passwd", "-fsck"}
+	for _, flag1 := range opFlags {
+		var flag2 string
+		for _, flag2 = range opFlags {
+			if flag1 == flag2 {
+				continue
+			}
+			args := []string{flag1, flag2, "/tmp"}
+			//t.Logf("testing %v", args)
+			cmd := exec.Command(test_helpers.GocryptfsBinary, args...)
+			err := cmd.Run()
+			exitCode := test_helpers.ExtractCmdExitCode(err)
+			if exitCode != exitcodes.Usage {
+				t.Fatalf("this should have failed with code %d, but returned %d",
+					exitcodes.Usage, exitCode)
+			}
+		}
+	}
+}
+
+func TestNoexec(t *testing.T) {
+	dir := test_helpers.InitFS(t)
+	mnt := dir + ".mnt"
+	err := os.Mkdir(mnt, 0700)
+	if err != nil {
+		t.Fatal(err)
+	}
+	test_helpers.MountOrFatal(t, dir, mnt, "-extpass=echo test", "-noexec")
+	defer test_helpers.UnmountPanic(mnt)
+	sh := mnt + "/x.sh"
+	content := `#!/bin/bash
+echo hello
+`
+	err = ioutil.WriteFile(sh, []byte(content), 0755)
+	if err != nil {
+		t.Fatal(err)
+	}
+	err = exec.Command(sh).Run()
+	exitCode := test_helpers.ExtractCmdExitCode(err)
+	if exitCode != int(syscall.EACCES) {
+		t.Errorf("got exitcode %d instead of EPERM (%d)", exitCode, syscall.EPERM)
+	}
+}
+
+// Test that a missing argument to "-o" triggers exit code 1.
+// See also cli_args_test.go for comprehensive tests of "-o" parsing.
+func TestMissingOArg(t *testing.T) {
+	cmd := exec.Command(test_helpers.GocryptfsBinary, "foo", "bar", "-o")
+	err := cmd.Run()
+	exitCode := test_helpers.ExtractCmdExitCode(err)
+	if exitCode != exitcodes.Usage {
+		t.Fatalf("this should have failed with code %d, but returned %d",
+			exitcodes.Usage, exitCode)
+	}
+}
+
+// -exclude must return an error in forward mode
+func TestExcludeForward(t *testing.T) {
+	dir := test_helpers.InitFS(t)
+	mnt := dir + ".mnt"
+	err := test_helpers.Mount(dir, mnt, false, "-extpass", "echo test", "-exclude", "foo")
+	if err == nil {
+		t.Errorf("-exclude in forward mode should fail")
+	}
+	t.Log(err)
+}
