@@ -23,6 +23,7 @@ import (
 	"syscall"
 	"testing"
 
+	"github.com/rfjakob/gocryptfs/internal/stupidgcm"
 	"github.com/rfjakob/gocryptfs/internal/syscallcompat"
 	"github.com/rfjakob/gocryptfs/tests/test_helpers"
 )
@@ -58,6 +59,9 @@ func TestMain(m *testing.M) {
 	// Make "testing.Verbose()" return the correct value
 	flag.Parse()
 	for _, testcase = range matrix {
+		if testcase.openssl == "true" && stupidgcm.BuiltWithoutOpenssl {
+			continue
+		}
 		if testing.Verbose() {
 			fmt.Printf("matrix: testcase = %#v\n", testcase)
 		}
@@ -68,7 +72,13 @@ func TestMain(m *testing.M) {
 		opts = append(opts, fmt.Sprintf("-aessiv=%v", testcase.aessiv))
 		opts = append(opts, fmt.Sprintf("-raw64=%v", testcase.raw64))
 		test_helpers.MountOrExit(test_helpers.DefaultCipherDir, test_helpers.DefaultPlainDir, opts...)
+		before := test_helpers.ListFds()
 		r := m.Run()
+		after := test_helpers.ListFds()
+		if len(before) != len(after) {
+			fmt.Printf("fd leak? before, after:\n%v\n%v\n", before, after)
+			os.Exit(1)
+		}
 		test_helpers.UnmountPanic(test_helpers.DefaultPlainDir)
 		if r != 0 {
 			os.Exit(r)
@@ -142,6 +152,7 @@ func TestTruncate(t *testing.T) {
 	if err != nil {
 		t.FailNow()
 	}
+	defer file.Close()
 	// Grow to two blocks
 	file.Truncate(7000)
 	test_helpers.VerifySize(t, fn, 7000)
@@ -226,6 +237,7 @@ func TestFallocate(t *testing.T) {
 	if err != nil {
 		t.FailNow()
 	}
+	defer file.Close()
 	wellKnown := isWellKnownFS(test_helpers.DefaultCipherDir)
 	fd := int(file.Fd())
 	nBytes := test_helpers.Du(t, fd)
@@ -329,7 +341,6 @@ func TestFallocate(t *testing.T) {
 		}
 	}
 	// Cleanup
-	file.Close()
 	syscall.Unlink(fn)
 	if !wellKnown {
 		// Even though most tests have been executed still, inform the user
@@ -344,6 +355,7 @@ func TestAppend(t *testing.T) {
 	if err != nil {
 		t.FailNow()
 	}
+	defer file.Close()
 	data := []byte("testdata123456789") // length 17
 	var buf bytes.Buffer
 	var hashWant string
@@ -377,6 +389,7 @@ func TestFileHoles(t *testing.T) {
 	if err != nil {
 		t.Errorf("file create failed")
 	}
+	defer file.Close()
 	foo := []byte("foo")
 	file.Write(foo)
 	file.WriteAt(foo, 4096)
@@ -405,10 +418,12 @@ func TestRmwRace(t *testing.T) {
 	if err != nil {
 		t.Fatalf("file create failed")
 	}
+	defer f1.Close()
 	f2, err := os.Create(fn)
 	if err != nil {
 		t.Fatalf("file create failed")
 	}
+	defer f2.Close()
 
 	oldBlock := bytes.Repeat([]byte("o"), 4096)
 
@@ -684,7 +699,10 @@ func TestLchown(t *testing.T) {
 }
 
 // Set nanoseconds by path, symlink
-func TestUtimesNanoSymlink(t *testing.T) {
+func Symlink(t *testing.T) {
+	if runtime.GOOS == "darwin" {
+		t.Skipf("MacOS \"touch\" does not support \"--no-dereference\"")
+	}
 	path := test_helpers.DefaultPlainDir + "/utimesnano_symlink"
 	err := os.Symlink("/some/nonexisting/file", path)
 	if err != nil {
@@ -708,17 +726,15 @@ type utimesTestcaseStruct struct {
 	out [2]syscall.Timespec
 }
 
-func compareUtimes(want [2]syscall.Timespec, actual [2]syscall.Timespec) error {
-	tsNames := []string{"atime", "mtime"}
-	for i := range want {
-		if want[i].Sec != actual[i].Sec {
-			return fmt.Errorf("Wrong %s seconds: want=%d actual=%d", tsNames[i], want[i].Sec, actual[i].Sec)
-		}
-		if want[i].Nsec != actual[i].Nsec {
-			return fmt.Errorf("Wrong %s nanoseconds: want=%d actual=%d", tsNames[i], want[i].Nsec, actual[i].Nsec)
-		}
+// compareTimespec return true if the two passed Timespec are identical.
+func compareTimespec(want syscall.Timespec, actual syscall.Timespec) bool {
+	if want.Sec != actual.Sec {
+		return false
 	}
-	return nil
+	if want.Nsec != actual.Nsec {
+		return false
+	}
+	return true
 }
 
 const _UTIME_OMIT = ((1 << 30) - 2)
@@ -728,8 +744,8 @@ const _UTIME_OMIT = ((1 << 30) - 2)
 func doTestUtimesNano(t *testing.T, path string) {
 	utimeTestcases := []utimesTestcaseStruct{
 		{
-			in:  [2]syscall.Timespec{{Sec: 50, Nsec: 0}, {Sec: 50, Nsec: 0}},
-			out: [2]syscall.Timespec{{Sec: 50, Nsec: 0}, {Sec: 50, Nsec: 0}},
+			in:  [2]syscall.Timespec{{Sec: 50, Nsec: 0}, {Sec: 51, Nsec: 0}},
+			out: [2]syscall.Timespec{{Sec: 50, Nsec: 0}, {Sec: 51, Nsec: 0}},
 		},
 		{
 			in:  [2]syscall.Timespec{{Sec: 1, Nsec: 2}, {Sec: 3, Nsec: 4}},
@@ -744,6 +760,10 @@ func doTestUtimesNano(t *testing.T, path string) {
 			out: [2]syscall.Timespec{{Sec: 7, Nsec: 8}, {Sec: 5, Nsec: 6}},
 		},
 	}
+	if runtime.GOOS == "darwin" {
+		// darwin neither supports UTIME_OMIT nor nanoseconds (!?)
+		utimeTestcases = utimeTestcases[:1]
+	}
 	for i, tc := range utimeTestcases {
 		err := syscall.UtimesNano(path, tc.in[:])
 		if err != nil {
@@ -754,9 +774,13 @@ func doTestUtimesNano(t *testing.T, path string) {
 		if err != nil {
 			t.Fatal(err)
 		}
-		err = compareUtimes(tc.out, extractAtimeMtime(st))
-		if err != nil {
-			t.Errorf("Testcase %d: %v", i, err)
+		want := tc.out
+		have := extractAtimeMtime(st)
+		if !compareTimespec(want[0], have[0]) {
+			t.Errorf("Testcase %d: atime: want=%+v, have=%+v", i, want[0], have[0])
+		}
+		if !compareTimespec(want[1], have[1]) {
+			t.Errorf("Testcase %d: mtime: want=%+v, have=%+v", i, want[1], have[1])
 		}
 	}
 }
@@ -773,11 +797,15 @@ func TestUtimesNano(t *testing.T) {
 
 // Set nanoseconds by fd
 func TestUtimesNanoFd(t *testing.T) {
+	if runtime.GOOS == "darwin" {
+		t.Skipf("MacOS does not have /proc")
+	}
 	path := test_helpers.DefaultPlainDir + "/utimesnanofd"
 	f, err := os.Create(path)
 	if err != nil {
 		t.Fatal(err)
 	}
+	defer f.Close()
 	procPath := fmt.Sprintf("/proc/self/fd/%d", f.Fd())
 	doTestUtimesNano(t, procPath)
 }
