@@ -6,6 +6,7 @@ import (
 	"encoding/hex"
 	"encoding/json"
 	"fmt"
+	"io"
 	"io/ioutil"
 	"log"
 	"net"
@@ -21,7 +22,7 @@ import (
 )
 
 // TmpDir will be created inside this directory
-const testParentDir = "/tmp/gocryptfs-test-parent"
+var testParentDir = "/tmp/gocryptfs-test-parent"
 
 // GocryptfsBinary is the assumed path to the gocryptfs build.
 const GocryptfsBinary = "../../gocryptfs"
@@ -42,7 +43,19 @@ var DefaultPlainDir string
 // DefaultCipherDir is TmpDir + "/default-cipher"
 var DefaultCipherDir string
 
+// SwitchTestParentDir changes testParentDir. This is used when you want
+// to perform tests on a special filesystem. For example, the xattr tests
+// cannot run on tmpfs and use /var/tmp instead of /tmp.
+func SwitchTestParentDir(newDir string) {
+	testParentDir = newDir
+	doInit()
+}
+
 func init() {
+	doInit()
+}
+
+func doInit() {
 	X255 = string(bytes.Repeat([]byte("X"), 255))
 
 	os.MkdirAll(testParentDir, 0700)
@@ -157,8 +170,22 @@ func Mount(c string, p string, showOutput bool, extraArgs ...string) error {
 
 	cmd := exec.Command(GocryptfsBinary, args...)
 	if showOutput {
-		cmd.Stderr = os.Stderr
-		cmd.Stdout = os.Stdout
+		// The Go test logic waits for our stdout to close, and when we share
+		// it with the subprocess, it will wait for it to close it as well.
+		// Use an intermediate pipe so the tests do not hang when unmouting
+		// fails.
+		pr, pw, err := os.Pipe()
+		if err != nil {
+			return err
+		}
+		// We can close the fd after cmd.Run() has executed
+		defer pw.Close()
+		cmd.Stderr = pw
+		cmd.Stdout = pw
+		go func() {
+			io.Copy(os.Stdout, pr)
+			pr.Close()
+		}()
 	}
 
 	return cmd.Run()
@@ -168,7 +195,7 @@ func Mount(c string, p string, showOutput bool, extraArgs ...string) error {
 func MountOrExit(c string, p string, extraArgs ...string) {
 	err := Mount(c, p, true, extraArgs...)
 	if err != nil {
-		fmt.Printf("mount failed: %v", err)
+		fmt.Printf("mount failed: %v\n", err)
 		os.Exit(1)
 	}
 }
@@ -185,17 +212,37 @@ func MountOrFatal(t *testing.T, c string, p string, extraArgs ...string) {
 func UnmountPanic(dir string) {
 	err := UnmountErr(dir)
 	if err != nil {
-		fmt.Println(err)
-		panic(err)
+		fmt.Printf("UnmountPanic: %v. Running lsof %s\n", err, dir)
+		cmd := exec.Command("lsof", dir)
+		cmd.Stdout = os.Stdout
+		cmd.Stderr = os.Stderr
+		cmd.Run()
+		panic("UnmountPanic: unmount failed: " + err.Error())
 	}
 }
 
-// UnmountErr tries to unmount "dir" and returns the resulting error.
-func UnmountErr(dir string) error {
-	cmd := exec.Command(UnmountScript, "-u", "-z", dir)
-	cmd.Stdout = os.Stdout
-	cmd.Stderr = os.Stderr
-	return cmd.Run()
+// UnmountErr tries to unmount "dir", retrying 10 times, and returns the
+// resulting error.
+func UnmountErr(dir string) (err error) {
+	max := 10
+	// When a new filesystem is mounted, Gnome tries to read files like
+	// .xdg-volume-info, autorun.inf, .Trash.
+	// If we try to unmount before Gnome is done, the unmount fails with
+	// "Device or resource busy", causing spurious test failures.
+	// Retry a few times to hide that problem.
+	for i := 1; i <= max; i++ {
+		cmd := exec.Command(UnmountScript, "-u", dir)
+		cmd.Stdout = os.Stdout
+		cmd.Stderr = os.Stderr
+		err = cmd.Run()
+		if err == nil {
+			return nil
+		}
+		code := ExtractCmdExitCode(err)
+		fmt.Printf("UnmountErr: got exit code %d, retrying (%d/%d)\n", code, i, max)
+		time.Sleep(100 * time.Millisecond)
+	}
+	return err
 }
 
 // Md5fn returns an md5 string for file "filename"
@@ -247,8 +294,10 @@ func TestMkdirRmdir(t *testing.T, plainDir string) {
 		t.Error(err)
 		return
 	}
-	// Removing a non-empty dir should fail with ENOTEMPTY
-	if os.Mkdir(dir, 0777) != nil {
+	// Create a directory and put a file in it
+	// Trying to rmdir it should fail with ENOTEMPTY
+	err = os.Mkdir(dir, 0777)
+	if err != nil {
 		t.Error(err)
 		return
 	}
@@ -261,13 +310,15 @@ func TestMkdirRmdir(t *testing.T, plainDir string) {
 	err = syscall.Rmdir(dir)
 	errno := err.(syscall.Errno)
 	if errno != syscall.ENOTEMPTY {
-		t.Errorf("Should have gotten ENOTEMPTY, go %v", errno)
+		t.Errorf("Should have gotten ENOTEMPTY, got %v", errno)
 	}
-	if syscall.Unlink(dir+"/file") != nil {
+	err = syscall.Unlink(dir + "/file")
+	if err != nil {
 		t.Error(err)
 		return
 	}
-	if syscall.Rmdir(dir) != nil {
+	err = syscall.Rmdir(dir)
+	if err != nil {
 		t.Error(err)
 		return
 	}
@@ -305,34 +356,41 @@ func TestRename(t *testing.T, plainDir string) {
 }
 
 // VerifyExistence checks in 3 ways that "path" exists:
-// stat, open, readdir
+// stat, open, readdir. Returns true if the path exists, false otherwise.
+// Panics if the result is inconsistent.
 func VerifyExistence(path string) bool {
 	// Check that file can be stated
+	stat := true
 	_, err := os.Stat(path)
 	if err != nil {
 		//t.Log(err)
-		return false
+		stat = false
 	}
 	// Check that file can be opened
+	open := true
 	fd, err := os.Open(path)
 	if err != nil {
 		//t.Log(err)
-		return false
+		open = false
 	}
 	fd.Close()
 	// Check that file shows up in directory listing
+	readdir := false
 	dir := filepath.Dir(path)
 	name := filepath.Base(path)
 	fi, err := ioutil.ReadDir(dir)
-	if err != nil {
-		//t.Log(err)
-		return false
-	}
-	for _, i := range fi {
-		if i.Name() == name {
-			return true
+	if err == nil {
+		for _, i := range fi {
+			if i.Name() == name {
+				readdir = true
+			}
 		}
 	}
+	// If the result is consistent, return it.
+	if stat == open && open == readdir {
+		return stat
+	}
+	log.Panicf("inconsistent result: stat=%v open=%v readdir=%v", stat, open, readdir)
 	return false
 }
 
@@ -374,3 +432,40 @@ func QueryCtlSock(t *testing.T, socketPath string, req ctlsock.RequestStruct) (r
 	json.Unmarshal(buf, &response)
 	return response
 }
+
+// ExtractCmdExitCode extracts the exit code from an error value that was
+// returned from exec / cmd.Run()
+func ExtractCmdExitCode(err error) int {
+	if err == nil {
+		return 0
+	}
+	// OMG this is convoluted
+	if err2, ok := err.(*exec.ExitError); ok {
+		return err2.Sys().(syscall.WaitStatus).ExitStatus()
+	}
+	if err2, ok := err.(*os.PathError); ok {
+		return int(err2.Err.(syscall.Errno))
+	}
+	log.Panicf("could not decode error %#v", err)
+	return 0
+}
+
+// ListFds lists our open file descriptors.
+// We use /dev/fd because it exists on both Linux and MacOS.
+func ListFds() []string {
+	f, err := os.Open("/dev/fd")
+	if err != nil {
+		log.Panic(err)
+	}
+	defer f.Close()
+	names, err := f.Readdirnames(0)
+	if err != nil {
+		log.Panic(err)
+	}
+	for i, n := range names {
+		// Note: Readdirnames filters "." and ".."
+		target, _ := os.Readlink("/dev/fd/" + n)
+		names[i] = n + "=" + target
+	}
+	return names
+}
